# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test;
BEGIN { plan tests => 13 };
use Video::ivtv;
my $ivtvObj = Video::ivtv->new();
if (!defined $ivtvObj)
{
  print "1 failed!  ivtvObj is undefined!\n";
}
else
{
  ok(1); # If we made it this far, we're ok.
}

my $dev = "/dev/video0";
open (my $tuner, "<$dev") or die "Error opening $dev!  $!\n";
my $tunerFD = fileno($tuner);
my ($oldwidth, $oldheight) = $ivtvObj->getResolution($tunerFD);
print "Width = $oldwidth, Height = $oldheight\n";
if ($oldwidth != -1 && $oldheight != -1)
{
	ok(2);
}
else
{
	print "2 failed!  Don't know how to trigger that.\n";
}

my $result = $ivtvObj->setResolution($tunerFD, 640, 480);
if (defined $result)
{
	if ($result)
	{
		my ($width, $height) = $ivtvObj->getResolution($tunerFD);
		if ($width == 640 && $height == 480)
		{
			ok(3);
      # restore the previous setting.
      $result = $ivtvObj->setResolution($tunerFD, $oldwidth, $oldheight);
		}
		else
		{
			print "Error:  width='$width', height='$height'\n";
			print "3 failed!  Don't know how to trigger that.\n";
		}
	}
	else
	{
		print "3 failed!  Don't know how to trigger that.\n";
	}
}
else
{
	print "3 failed!  Don't know how to trigger that.\n";
}

my $standard = $ivtvObj->getStandard($tunerFD);
if ($standard == 0x00003000 || $standard == 0x000000FF || $standard == 0x007F0000)
{
  ok(4);
}
else
{
  print "4 failed!  Standard = '$standard'!\n";
}

# set the Video Standard to NTSC (0x3000)
$result = $ivtvObj->setStandard($tunerFD, 0x00003000);
if ($result)
{
  my $newStandard = $ivtvObj->getStandard($tunerFD);
  if ($newStandard == $standard)
  {
    # restore the previous standard.
    $ivtvObj->setStandard($tunerFD, $standard);
    ok(5);
  }
}
else
{
	print "5 failed!  Don't know how to trigger that.\n";
}

my $freq = $ivtvObj->getFrequency($tunerFD, 0);
if ($freq != 0)
{
  ok(6);
}
else
{
  print "6 failed!  Frequency = '$freq'!\n";
}

# set the Video Frequency to 8276 (SCiFI)
$result = $ivtvObj->setFrequency($tunerFD, 0, 8276);
if ($result)
{
  # verify the frequency got set to what we want.
  my $newfreq = $ivtvObj->getFrequency($tunerFD, 0);
  if ($newfreq == 8276)
  {
    # restore the previous frequency value
    $result = $ivtvObj->setFrequency($tunerFD, 0, $freq);
    ok(7);
  }
  else
  {
    print "7 failed!  Frequency = '$freq' instead of 8276!\n";
  }
}
else
{
	print "7 failed!  Don't know how to trigger that.\n";
}

# get the Video Input
my $input = $ivtvObj->getInput($tunerFD);
if ($input >= 0)
{
  print "input = '$input'\n";
  ok(8);
}
else
{
  print "8 failed!  Don't know how to trigger that.\n";
}

# set the Video Input
$result = $ivtvObj->setInput($tunerFD, 4); # Tuner on my card.
if ($result)
{
  # verify the input got set to what I wanted.
  my $newInput = $ivtvObj->getInput($tunerFD);
  if ($newInput != 4)
  {
    print "9 failed!  Input = '$newInput' instead of 4!\n";
  }
  else
  {
    # reset the input back to what it was originally.
    $ivtvObj->setInput($tunerFD, $input);
    ok(9);
  }
}
else
{
  print "9 failed!  Don't know how to trigger that.\n";
}

# enumerate the available Standards
my $done = 0;
my $index;
for ($index=0; !$done; ++$index)
{
  my ($i, $id, $name, $numerator, $denominator, $framelines) = $ivtvObj->enumerateStandard($tunerFD, $index);
  if ($i == -1)
  {
    $done = 1;  # this index is invalid.
  }
  else
  {
    printf("%d 0x%08x %s %d/%d %d\n", $i, $id, $name, $numerator, $denominator, $framelines);
  }
}
if ($index > 0)
{
  ok(10);
}
else
{
  print "10 failed!  Don't know how to trigger that.\n";
}

# enumerate the available Inputs
$done = 0;
$index;
for ($index=0; !$done; ++$index)
{
  my ($i, $name, $type, $audioset, $tuner, $std, $status) = $ivtvObj->enumerateInput($tunerFD, $index);
  if ($i == -1)
  {
    $done = 1;  # this index is invalid.
  }
  else
  {
    printf("%d %s 0x%08x 0x%08x 0x%08x 0x%08x %d\n", $i, $name, $type, $audioset, $tuner, $std, $status);
  }
}
if ($index > 0)
{
  ok(11);
}
else
{
  print "11 failed!  Don't know how to trigger that.\n";
}

# get the current codec settings
my @codec = $ivtvObj->getCodecInfo($tunerFD);
if (scalar @codec == scalar(keys %{$ivtvObj->{codecIndexes}}))
{
  print "aspect: $codec[$ivtvObj->{codecIndexes}{aspect}]\naudio: $codec[$ivtvObj->{codecIndexes}{audio}]\nbframes: $codec[$ivtvObj->{codecIndexes}{bframes}]\n";
  print "bitrate: $codec[$ivtvObj->{codecIndexes}{bitrate}]\nbitrate_peak: $codec[$ivtvObj->{codecIndexes}{bitrate_peak}]\ndnr_mode: $codec[$ivtvObj->{codecIndexes}{dnr_mode}]\n";
  print "dnr_spatial: $codec[$ivtvObj->{codecIndexes}{dnr_spatial}]\ndnr_temporal: $codec[$ivtvObj->{codecIndexes}{dnr_temporal}]\ndnr_type: $codec[$ivtvObj->{codecIndexes}{dnr_type}]\n";
  print "framerate: $codec[$ivtvObj->{codecIndexes}{framerate}]\nframespergop: $codec[$ivtvObj->{codecIndexes}{framespergop}]\ngop_closure: $codec[$ivtvObj->{codecIndexes}{gop_closure}]\n";
  print "pulldown: $codec[$ivtvObj->{codecIndexes}{pulldown}]\nstream_type: $codec[$ivtvObj->{codecIndexes}{stream_type}]\n";
  ok(12);
}
else
{
  print "12 failed!  getCodecInfo() returned " . int(@codec) . " values!\n";
}

# test setting them, then restore
my @newCodec = $ivtvObj->getCodecInfo($tunerFD);
$newCodec[$codecIndexes{bitrate}] = 6500000; # change the bitrate value
$newCodec[$codecIndexes{bitrate_peak}] = 9000000; # change the bitrate_peak value
$result = $ivtvObj->setCodecInfo($tunerFD, @newCodec);
if ($result)
{
  # verify that the values are what I set.
  my @testCodec = $ivtvObj->getCodecInfo($tunerFD);
  if ($testCodec[$ivtvObj->{codecIndexes}{bitrate}] == $newCodec[$ivtvObj->{codecIndexes}{bitrate}] && $testCodec[$ivtvObj->{codecIndexes}{bitrate_peak}] == $newCodec[$ivtvObj->{codecIndexes}{bitrate_peak}])
  {
    # restore the original values.
    $result = $ivtvObj->setCodecInfo($tunerFD, @codec);
    ok(13);
  }
  else
  {
    print "13 failed!  getCodecInfo() returned bitrate = '$testCodec[$ivtvObj->{codecIndexes}{bitrate}]', bitrate_peak = '$testCodec[$ivtvObj->{codecIndexes}{bitrate_peak}]'\n";
  }
}
else
{
  print "13 failed!  Don't know how to trigger that.\n";
}


#########################

# Insert your test code below, the Test module is use()ed here so read
# its man page ( perldoc Test ) for help writing this test script.

